#! /usr/bin/env python

"""QuickWeb

Usage:
    quickweb create <app_directory> <template_name> [--force]
    quickweb run [<app_directory>] [-l <listener_address>] [--no-logs]
    quickweb setup-cf-deployment [<app_directory>]
    quickweb setup-docker-deployment [<app_directory>]
    quickweb describe [<app_directory>]
    quickweb templates

Examples:

    quickweb create my-quickweb-app bootstrap-navbar-fixed-top

Options:
  -h --help     Show this screen.
"""
import os
import sys
from os.path import join, dirname
from docopt import docopt
from colorama import init
from . import app_manager, template_manager, version

init()

# Make sure we use the source directory for imports when running during development
script_dir = join(dirname(os.path.realpath(__file__)), "..")
sys.path.insert(0, script_dir)


def main():
    arguments = docopt(__doc__, version="QuickWeb %s" % version())
    print()

    if arguments["create"]:
        app_manager.create(
            arguments["<app_directory>"],
            arguments["<template_name>"],
            arguments["--force"],
        )

    if arguments["run"]:
        app_manager.run(
            arguments["<app_directory>"],
            arguments["<listener_address>"],
            arguments["--no-logs"],
        )

    if arguments["setup-cf-deployment"]:
        app_manager.setup_cf_deployment(arguments["<app_directory>"])

    if arguments["setup-docker-deployment"]:
        app_manager.setup_docker_deployment(arguments["<app_directory>"])

    if arguments["describe"]:
        app_manager.describe(arguments["<app_directory>"])

    if arguments["templates"]:
        template_manager.list()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from os.path import join, basename
from quickweb import controller, template
from glob import glob
from time import time
import imp
import cherrypy
import os
import logging.config
import warnings


# Set a default app root handler, to bind controllers until we find an
# "index" controller
class TemplateRenderer(object):  # Application Server Root place holder
    @controller.publish
    def index(self):
        path = controller.controller_path()[1]
        if path == "":
            html_template_filename = "index.html"
        else:
            html_template_filename = path + ".html"
        return template.render(html_template_filename)


def setup_logging():
    LOG_CONF = {
        "version": 1,
        "formatters": {
            "void": {"format": ""},
            "standard": {"format": "%(asctime)s [%(levelname)s] %(name)s: %(message)s"},
        },
        "handlers": {
            "default": {
                "level": "INFO",
                "class": "logging.StreamHandler",
                "formatter": "standard",
                "stream": "ext://sys.stdout",
            },
            "cherrypy_console": {
                "level": "INFO",
                "class": "logging.StreamHandler",
                "formatter": "void",
                "stream": "ext://sys.stdout",
            },
            "cherrypy_access": {
                "level": "INFO",
                "class": "logging.handlers.RotatingFileHandler",
                "formatter": "void",
                "filename": "access.log",
                "maxBytes": 10485760,
                "backupCount": 20,
                "encoding": "utf8",
            },
            "cherrypy_error": {
                "level": "INFO",
                "class": "logging.handlers.RotatingFileHandler",
                "formatter": "void",
                "filename": "errors.log",
                "maxBytes": 10485760,
                "backupCount": 20,
                "encoding": "utf8",
            },
        },
        "loggers": {
            "": {"handlers": ["default"], "level": "INFO"},
            "db": {"handlers": ["default"], "level": "INFO", "propagate": False},
            "cherrypy.access": {
                "handlers": ["cherrypy_access"],
                "level": "INFO",
                "propagate": False,
            },
            "cherrypy.error": {
                "handlers": ["cherrypy_console", "cherrypy_error"],
                "level": "INFO",
                "propagate": False,
            },
        },
    }
    logging.config.dictConfig(LOG_CONF)


def set_default_config():

    # Enforce utf8 encoding
    cherrypy.config.update(
        {
            "tools.encode.on": True,
            "tools.encode.encoding": "utf-8",
            "tools.encode.errors": "replace",
        }
    )
    cherrypy.config.update({"tools.sessions.on": True})
    cherrypy.config.update({"environment": "embedded"})
    setup_logging()


# Load and setup controllers
def load_controllers(app_name, controller_dir):
    print("*** Checking for controllers")
    for controller_file in glob(join(controller_dir, "*.py")):
        module_name = app_name + ".controllers." + basename(controller_file)
        controller_path = basename(controller_file).split(".")[0]
        print("- Loading controller %s -> %s " % (module_name, controller_path))
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", RuntimeWarning)
            handler = imp.load_source(module_name, controller_file)
        controller.attach(controller_path, handler.Controller())


def set_template_dirs(template_dirs, webroot_directory):
    for template_dirname in template_dirs:
        print("- Setting template directory", template_dirname)
    template.set_directories(webroot_directory, template_dirs)


def set_root_appnodes(webroot_directory):
    for html_filename in glob(join(webroot_directory, "*.html")):
        node_name = html_filename.split("/")[-1].split(".")[0]
        if node_name.startswith("_"):  # Dot not render template elements
            continue
        if node_name == "index":  # Index is already handled by the root dir
            continue
        controller.attach("/" + node_name, TemplateRenderer())


# Setup static file serve handling for 'static' sub-dirs
def set_static_dirs(application, static_dirs):
    print("*** Checking for static dirs")
    for location in static_dirs:
        for static_dir in os.listdir(location):
            full_static_dir = join(location, static_dir)
            print("- Setting static dir %s -> %s" % (static_dir, full_static_dir))
            conf = {}
            conf["/" + basename(static_dir)] = {
                "tools.staticdir.dir": full_static_dir,
                "tools.staticdir.on": True,
            }
            application.merge(conf)


def setup(app_name, app_directory):

    quickweb_dir = os.path.dirname((os.path.realpath(__file__)))
    version_filename = os.path.join(quickweb_dir, "..", "version")
    with open(version_filename) as version_file:
        version = version_file.readline().strip("\r\n")

    application_root = controller.set_approot(TemplateRenderer())
    app = cherrypy.Application(application_root, script_name=None, config=None)

    set_default_config()

    start_t = time()
    print(
        "=" * 10
        + " Starting app %s using QuickWeb %s " % (app_name, version)
        + "=" * 10
    )
    print("= Startup location", app_directory)
    static_dirs = []
    webroot_directory = join(app_directory, "webroot")
    for location in ["template", "webroot"]:
        for root, dirs, files in os.walk(join(app_directory), location):
            for name in dirs:
                if name == "static":
                    static_directory = join(join(root, name))
                    if static_directory not in static_dirs:
                        static_dirs.append(static_directory)

    set_static_dirs(app, static_dirs)
    set_root_appnodes(webroot_directory)
    load_controllers(app_name, join(app_directory, "controllers"))
    stop_t = time()
    print("=" * 10 + " Startup completed in %0.3f ms " % ((stop_t - start_t) * 1000.0))

    return app



# -*- coding: utf-8 -*-
import os
import re
import sys
import cherrypy
import importlib
from os.path import join, exists, basename, splitext
from glob import glob

this = sys.modules[__name__]


class ApplicationNode:
    pass


this._app_root = ApplicationNode()

# Dictionary with URL to Name mapping to be displayed on navigation components
this._navigation = {}

# List of "place_after" constraints
this._navigation_place_after = []


def set_app_root(controller_obj):
    for key, value in this._app_root.__dict__.items():
        setattr(controller_obj, key, value)
    this._app_root = controller_obj


def get_app_root():
    return this._app_root


def find_path_base(controller_path_name):
    controller_path_name = controller_path_name.strip(os.sep)
    path_parts = controller_path_name.split(os.sep)

    path_root = this._app_root

    path_to_resource = path_parts[:-1]
    for part_name in path_to_resource:
        new_path_root = getattr(path_root, part_name, None)
        path_root = new_path_root
    return path_root


def attach(controller_path_name, controller_obj):
    """ Binds a controller object to an application URL """

    # Map OS path separator to URL separators (Needed on Windows)
    controller_path_name = controller_path_name.replace(os.sep, "/")

    controller_path_name = controller_path_name.strip("/")
    path_parts = controller_path_name.split("/")

    #  print('attach', controller_path_name, controller_obj, len(this.app_root.__dict__))
    path_root = this._app_root

    # If a path
    resource_name = path_parts[-1]
    path_to_resource = path_parts[:-1]
    for part_name in path_to_resource:
        new_path_root = getattr(path_root, part_name, None)
        if new_path_root is None:
            new_path_root = ApplicationNode()
            setattr(path_root, part_name, new_path_root)
        path_root = new_path_root

    # If we get a new root index, migrate current root resources to the new one
    if controller_path_name == "":
        set_app_root(controller_obj)
    else:
        setattr(path_root, resource_name, controller_obj)


def publish(func=None, *args):
    return cherrypy.expose(func, *args)


def redirect(url):
    raise cherrypy.HTTPRedirect(url)


def get_cookie(name, default_value=None):
    def unescape(s):
        m = re.match(r'^"(.*)"$', s)
        s = m.group(1) if m else s
        return s.replace("\\\\", "\\")

    if name in cherrypy.request.cookie:
        return unescape(cherrypy.request.cookie[name].value).decode("unicode-escape")
    else:
        return default_value


def get_session_value(name, default_value=None):
    if name in cherrypy.session:
        return cherrypy.session[name]
    else:
        return default_value


def set_session_value(name, value):
    cherrypy.session[name] = value


def set_cookie(name, value, path="/", max_age=3600, version=1):
    cookie = cherrypy.response.cookie
    cookie[name] = value.encode("unicode-escape")
    cookie[name]["path"] = path
    cookie[name]["max-age"] = max_age
    cookie[name]["version"] = version


def delete_cookie(name):
    cherrypy.response.cookie[name] = "deleting"
    cherrypy.response.cookie[name]["expires"] = 0


def set_response(name, value):
    cherrypy.response.headers[name] = value


def method():
    return cherrypy.request.method


def params():
    return cherrypy.request.params


def controller_url():
    """
    @return: the current request url
    """
    return cherrypy.request.path_info


def controller_path():
    """
    @return: the current request path
    """
    return cherrypy.request.path_info.split("/")


def get_config():
    return cherrypy.config


def navigation_elements(start_path=None, using_placement=True):
    """
    Returns a list with { 'name': name, 'link': link}
    for each renderable ellement found on path
    """
    app_node = get_app_root()
    _navigation = app_node.__dict__.get("_navigation")
    if _navigation:
        return _navigation

    nav_list = []
    for leaf_name, leaf_obj in get_app_root().__dict__.iteritems():
        contains_index = hasattr(leaf_obj, "index")
        if contains_index and leaf_name[0] != "_":
            nav_name = this._navigation.get(leaf_name, leaf_name)
            nav_list.append({"name": nav_name, "link": leaf_name})

    if using_placement:
        for constraint_source, constraint_target in this._navigation_place_after:
            source_pos = [
                i for i, x in enumerate(nav_list) if x["link"] == constraint_source
            ][0]
            target_pos = [
                i for i, x in enumerate(nav_list) if x["link"] == constraint_target
            ][0]
            if source_pos < target_pos:
                swapping_element = nav_list.pop(source_pos)
                nav_list.insert(target_pos, swapping_element)

    return nav_list


def current_path():
    return controller_path()[1]


def helpers():
    """ Returns a dict with objects that should be available to templates """
    helpers_dict = {
        "navigation_elements": navigation_elements,
        "current_path": current_path,
        "language": get_lang,
        "domain": get_domain,
        "scheme": get_scheme,
    }
    return helpers_dict


def get_host():
    for key, value in cherrypy.request.header_list:
        if key.upper() == "HOST":
            return value.lower()


def get_lang():
    host = get_host()
    if host:
        return host.split(".")[0]


def get_domain():
    host = get_host()

    if host:
        if "." in host:
            domain = ".".join(host.split(".")[1:])
        else:
            domain = host
        return domain


def get_scheme():
    return cherrypy.request.scheme


def set_navigation_info(*args, **kwargs):
    #  cherrypy.(str(args))
    #  cherrypy.log(str(kwargs))

    cherrypy.log("Setting navigation info %s %s" % (str(args), str(kwargs)))

    valid_kwargs = ["name", "place_after"]
    for k in kwargs:
        if k not in valid_kwargs:
            raise Exception("Unsupported parameter %s" % k)

    if "name" in kwargs:
        this._navigation[current_path()] = kwargs["name"]

    # While the code provides support for multiple types, for simplicity only 'place_after'
    # is implemented.
    constraint_target = kwargs.get("place_after")
    if constraint_target[0] != "/":
        constraint_target = "/" + constraint_target

    if not constraint_target:
        return

    was_found = [
        x
        for x in navigation_elements(using_placement=False)
        if x["link"] == constraint_target
    ]

    if not was_found:
        cherrypy.log("Unknown target " + constraint_target)
        return

    was_found = [x for x in this._navigation_place_after if x[2] == constraint_target]

    if was_found:
        raise Exception("Duplicated place_after for target '%s'" % constraint_target)

    this._navigation_place_after((current_path(), constraint_target))


def load_app_modules(app_directory):
    libglob = join(app_directory, "lib", "*.py")
    libinit_fname = join(app_directory, "lib", "__init__.py")
    if not exists(libinit_fname):
        return

    spec = importlib.util.spec_from_file_location("lib", libinit_fname)
    lib_load_module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(lib_load_module)
    setattr(this, "lib", lib_load_module)

    for filename in glob(libglob):
        print("** Loading library", filename)
        module_name = splitext(basename(filename))[0]
        spec = importlib.util.spec_from_file_location(module_name, filename)
        load_module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(load_module)
        setattr(lib_load_module, module_name, load_module)
        cherrypy.engine.autoreload.files.add(filename)



"""
"""
import os
import sys
from time import time
from os.path import join, exists
from glob import glob
import cherrypy
import quickweb
from quickweb import startup
from quickweb.colorhelper import info
import importlib


def run(app_directory, listener_address=None, no_logs=False, running_describe=False):
    """
    When an application is run, the following is performed:

    - Identify application root
        - Check for qwstart.py on
            - Startup script directory
            - Current working directory

    - Setup port number, if $PORT is not set, use a random port
    """
    start_t = time()  # Use for startup time calculation
    print(
        "** Starting application %s using QuickWeb %s "
        % (info(app_directory), info(quickweb.version()))
    )
    startup_cwd = os.getcwd()

    # Check if beeing run from gunicorn
    is_gunicorn = "gunicorn" in os.environ.get("SERVER_SOFTWARE", "")
    if is_gunicorn:
        sys.stderr.write(
            "Quickweb provides it's own HTTP server module.\n"
            "Running from another HTTP server is not supported at this time\n"
        )
        sys.exit(1)

    # Identify the application root directory
    app_root_directory = app_directory or os.getcwd()

    dbsupport_fname = join(app_root_directory, "dbsupport.py")
    dbsupport = None
    if exists(dbsupport_fname):
        spec = importlib.util.spec_from_file_location("dbsupport", dbsupport_fname)
        dbsupport = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(dbsupport)

    startup.setup_app("app_name", app_root_directory, no_logs)

    if running_describe:
        return

    colored_elapsed_time = info("%0.2fms" % ((time() - start_t) * 1000.0))
    print("=" * 10 + " Startup completed in " + colored_elapsed_time)

    # Determine the HTTP listener port
    listener_port = int(os.getenv("PORT", 8080))
    if os.name == "posix":
        socket_host = "0.0.0.0"
    else:
        socket_host = "127.0.0.1"
    if listener_address is not None:
        socket_host = listener_address
    cherrypy.config.update({"server.socket_host": socket_host})
    cherrypy.config.update({"server.socket_port": listener_port})

    # In some platforms signals are not available:
    if hasattr(cherrypy.engine, "signals"):
        cherrypy.engine.signals.subscribe()
    cherrypy.engine.subscribe("stop", lambda: os.chdir(startup_cwd))
    if dbsupport:
        print("** Setting up datbase")
        cherrypy.engine.subscribe("start", dbsupport.setup_database)
        cherrypy.engine.subscribe("stop", dbsupport.cleanup_database)

    cherrypy.engine.start()
    cherrypy.engine.block()



#!/bin/python

import os
import sys
import time
from glob import glob
from os.path import join
import subprocess

TEST_PORT = 8999

os.environ["PORT"] = str(TEST_PORT)
os.environ["TEST_MODE"] = "1"


test_specs = glob(join("test-app", "tests", "*.yaml"))


subp = subprocess.Popen(
    [sys.executable, "-m", "quickweb", "run", "test-app"], close_fds=True
)

print("Waiting 1s for the server warmup")
# Allow 1s for the process to start
time.sleep(1)


for test_filename in test_specs:
    http_unit_test = join("quickweb", "http_unit_test.py")
    cmd = sys.executable + " %s http://127.0.0.1:%d %s" % (
        http_unit_test,
        TEST_PORT,
        test_filename,
    )
    rc = os.system(cmd)
    if rc != 0:
        subp.terminate()
        exit(2)

subp.terminate()



"""Installs QuickWeb using distutils

Run:
    python setup.py install

to install this package.
"""
from setuptools import setup, find_packages
from os.path import join

name = "QuickWeb"

desc = "Rapid Development Python Web Framework"
long_desc = "QuickWeb is a rapid web development python framework"


classifiers = """
Development Status :: 1 - Planning
Intended Audience :: Developers
License :: Freely Distributable
License :: OSI Approved :: BSD License
Operating System :: OS Independent
Programming Language :: Python :: 3.7
Topic :: Internet :: WWW/HTTP :: WSGI
Topic :: Internet :: WWW/HTTP :: WSGI :: Application
Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware
Topic :: Internet :: WWW/HTTP :: WSGI :: Server
"""

requirements = open("requirements.txt").read()

setup(
    name=name,
    version=open(join("quickweb", "version")).readline().strip("\r\n"),
    description=desc,
    long_description=long_desc,
    author="João Pinto",
    author_email="lamego.pinto@gmail.com",
    classifiers=[x for x in classifiers.splitlines() if x],
    install_requires=[x for x in requirements.splitlines() if x],
    url="https://github.com/OpenPipe/QuickWeb",
    packages=find_packages(),
    package_data={"quickweb.doc": ["*"]},
    include_package_data=True,
    entry_points={"console_scripts": ["quickweb = quickweb.__main__:main"]},
)
